library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;
--
-- Copyright (C) 2007, Peter C. Wallace, Mesa Electronics
-- http://www.mesanet.com


entity i43shim is
	generic
		(  
			FastClk : boolean := true;
			PortWidth : integer := 24;
			Ports : integer := 2;
			I30Pinout : boolean := true;
			SDMCIO : boolean := true;
			Naxis : integer := 4;
			StartingI30Conn : integer := 0;
			ConnswithI30 : integer := 1;
			StartingIOConn : integer := 1;
			ConnsWithIO : integer := 1
			
						

		);
		
	Port (	CLK : in std_logic;
				LEDS : out std_logic_vector(7 downto 0);
				IOBITS : inout std_logic_vector((PortWidth*Ports) -1 downto 0);
				USB_DATABUS : inout std_logic_vector(7 downto 0);
				USB_WRITE : out std_logic;
				USB_RD :out std_logic;
				USB_TXE : in std_logic;
				USB_RXF : in std_logic;
				RECONFIG : out std_logic;
				PARACONFIG : out std_logic;
				SPICLK : out std_logic;
				SPIIN : in std_logic;
				SPIOUT : out std_logic;
				SPICS : out std_logic
		 );
end i43shim;

architecture Behavioral of i43shim is
	 
-- GPIO interface signals
signal ReconfigSel : std_logic;
signal ReConfigreg : std_logic := '0';

signal LoadSPIReg : std_logic;
signal ReadSPIReg : std_logic;
signal LoadSPICS : std_logic;
signal ReadSPICS : std_logic;

signal LoadUSBDataReg : std_logic;
signal ReadUSBData : std_logic;
signal LoadUSBControlReg : std_logic;
signal ReadUSBStatus : std_logic;
signal USBDataReg : std_logic_vector(7 downto 0);
signal USBContReg : std_logic_vector(2 downto 0) := "011";
alias  USB_RdReg : std_logic is USBContReg(0);
alias  USB_WriteReg : std_logic is USBContReg(1);
alias  USB_TSEn : std_logic is USBContReg(2);


signal  iabus : std_logic_vector(9 downto 0);		  -- program address bus
signal  idbus : std_logic_vector(15 downto 0);		  -- program data bus		 
signal  mradd : std_logic_vector(11 downto 0);	  -- memory read address
signal  ioradd :  std_logic_vector(11 downto 0);	-- I/O read address
signal  mwadd : std_logic_vector(11 downto 0);	  -- memory write address
signal  mibus : std_logic_vector(7 downto 0);		  -- memory data in bus	  
signal  mobus : std_logic_vector(7 downto 0);		  -- memory data out bus
signal  mwrite : std_logic;								  -- memory write signal		  
signal  mread : std_logic;								     -- memory read signal	
signal pagedmradd : std_logic_vector(10 downto 0);
signal pagedmwadd : std_logic_vector(10 downto 0);
signal pagedmwrite : std_logic;

signal mibus_ram : std_logic_vector(7 downto 0);		  -- memory data in bus RAM
signal mibus_io : std_logic_vector(7 downto 0);			  -- memory data in bus IO

alias wiosel : std_logic is mwadd(10); 
alias riosel : std_logic is ioradd(10);

signal wseladd: std_logic_vector(7 downto 0); 
signal rseladd: std_logic_vector(7 downto 0); 
signal clkfx : std_logic;
signal clk0 : std_logic;
signal fclk : std_logic;

signal portadata : std_logic_vector (11 downto 0);
signal portbdata : std_logic_vector (11 downto 0);
signal portcdata : std_logic_vector (11 downto 0);
signal portddata : std_logic_vector (11 downto 0);
signal portedata : std_logic_vector (11 downto 0);
signal portfdata : std_logic_vector (11 downto 0);
signal int: std_logic;
-- on card debug leds
signal SDMCLEDs      :   std_logic_vector(7 downto 0);
signal LEDLatch      :   std_logic_vector(7 downto 0);
-- motor control stuff
signal a         :    std_logic_vector (naxis -1 downto 0);
signal b         :    std_logic_vector (naxis -1 downto 0);
signal idx       :    std_logic_vector (naxis -1 downto 0);
signal pwma       : std_logic_vector (naxis -1 downto 0);
signal dira       : std_logic_vector (naxis -1 downto 0);
signal pwmb       : std_logic_vector (naxis -1 downto 0);
signal dirb       : std_logic_vector (naxis -1 downto 0);
signal pwmc       : std_logic_vector (naxis -1 downto 0);
signal dirc       : std_logic_vector (naxis -1 downto 0);
signal ena        :   std_logic_vector (naxis -1 downto 0);
-- 7i32 signals only
signal enaa       :   std_logic_vector (naxis -1 downto 0);
signal enab       :   std_logic_vector (naxis -1 downto 0);
signal enac       :   std_logic_vector (naxis -1 downto 0);
signal cca        :   std_logic_vector (naxis -1 downto 0);
signal ccb        :   std_logic_vector (naxis -1 downto 0);
signal altidx     :   std_logic_vector (naxis -1 downto 0);

-- 7I39 signals only
signal HALL0     :   std_logic_vector (4 downto 0);
signal HALL1     :   std_logic_vector (4 downto 0);
signal HALL2     :   std_logic_vector (4 downto 0);
signal HALL3     :   std_logic_vector (4 downto 0);
signal HALL4     :   std_logic_vector (4 downto 0);
signal HALL5     :   std_logic_vector (4 downto 0);
signal HALL6     :   std_logic_vector (4 downto 0);
signal HALL7     :   std_logic_vector (4 downto 0);

signal SDMCDataIn: std_logic_vector(15 downto 0);
signal SDMCDataOut: std_logic_vector(15 downto 0);
signal SDMCReadStb: std_logic;
signal SDMCTSEn: std_logic;
signal SDMCWriteStb: std_logic;
signal SDMCSelect: std_logic;
signal SDMCRAddr: std_logic_vector(1 downto 0);
signal SDMCWAddr: std_logic_vector(1 downto 0);
signal SDMCLowDataLatch: std_logic_vector(7 downto 0);

begin

  DoubledClock: if (FastClk) generate
  
   ClockMult : DCM
		generic map (
			CLKDV_DIVIDE => 2.0,
			CLKFX_DIVIDE => 5, 
			CLKFX_MULTIPLY =>8,			-- 3/2 *50 FOR 75 MHz, 4/2 for 100 MHz, 8/5 for 80MHz, 7/4 for 87.5MHz
			CLKIN_DIVIDE_BY_2 => FALSE, 
			CLKIN_PERIOD => 20.0,          
			CLKOUT_PHASE_SHIFT => "NONE", 
			CLK_FEEDBACK => "1X",         
			DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", 
															
			DFS_FREQUENCY_MODE => "LOW",
			DLL_FREQUENCY_MODE => "LOW",
			DUTY_CYCLE_CORRECTION => TRUE,
			FACTORY_JF => X"C080",
			PHASE_SHIFT => 0, 
			STARTUP_WAIT => FALSE)
		port map (
	
			CLK0 => clk0,   	-- 
			CLKFB => clk0,  	-- DCM clock feedback
			CLKFX => clkfx,
			CLKIN => CLK,    	-- Clock input (from IBUFG, BUFG or DCM)
			PSCLK => '0',   	-- Dynamic phase adjust clock input
			PSEN => '0',     	-- Dynamic phase adjust enable input
			PSINCDEC => '0', 	-- Dynamic phase adjust increment/decrement
			RST => '0'        -- DCM asynchronous reset input
		);
  
	BUFG_inst : BUFG
		port map (
			O => fclk,    		-- Clock buffer output
			I => clkfx      	-- Clock buffer input
		);

  -- End of DCM_inst instantiation
	end generate;

	SimpleClock: if not FastClk generate
		clksource : process (CLK)
		begin
			fclk <= CLK;
		end process;
	end generate;
	



	asimplspi: entity simplespi
		generic map
		(
			buswidth => 8,
			div => 2,
			bits => 8
		)	
		port map 
		( 
			clk  => fclk,
			ibus  => mobus,
			obus  => mibus_io,
			loaddata  => LoadSPIReg,
			readdata  => ReadSPIReg,
			loadcs  => LoadSPICS,
			readcs  => ReadSPICS,
			spiclk  => SPIClk,
			spiin  => SPIIn,
			spiout  => SPIOut,
			spics  =>SPICS 
		 );


	processor: entity DumbAss8
	
	port map (
		clk		 => fclk,
		reset	  => '0',
		iabus	  =>  iabus,		  -- program address bus
		idbus	  =>  idbus,		  -- program data bus		 
		mradd	  =>  mradd,		  -- memory read address
		mwadd	  =>  mwadd,		  -- memory write address
		mibus	  =>  mibus,		  -- memory data in bus	  
		mobus	  =>  mobus,		  -- memory data out bus
		mwrite  =>  mwrite,		  -- memory write signal	
      mread   =>  mread		     -- memory read signal	
--		carryflg  =>				  -- carry flag
		);


	ASoftDMC: entity SoftDMC
	port map (
		 
	   DATABUSIN =>  SDMCDataIn,
		DATABUSOUT =>  SDMCDataOut,
		READSTB =>  SDMCReadStb,
		READENA =>  SDMCTSEn,
		WRITESTB => SDMCWriteStb,
		RADDR =>  SDMCRAddr,
		WADDR =>  SDMCWaddr,
		INTERFACECLK =>  fclk,
      SOFTDMCCLK  =>   fclk,
		INT =>  int,
      -- I/O Port bits
      PORTADATA =>   portadata,
      PORTBDATA =>   portbdata,
      PORTCDATA =>   portcdata,
      PORTDDATA =>   portddata,
      PORTEDATA =>   portedata,
      PORTFDATA =>   portfdata,
		      -- on card debug LEDS
      LEDS      =>   SDMCLEDs,
      -- motor control stuff
      A         =>    a,
      B         =>    b,
      IDX       =>    idx,
      PWMA       =>   pwma,
      DIRA       =>   dira,
      PWMB       =>   pwmb,
      DIRB       =>   dirb,
      PWMC       =>   pwmc,
      DIRC       =>   dirc,
		ENA        =>   ena,
-- 7I32/3PK signals only
		ENAA       =>   enaa,
		ENAB       =>   enab,
		ENAC       =>   enac,
		CCA        =>   cca,
		CCB        =>   ccb,
		ALTIDX     =>   altidx,

-- 7I39 signals only
		HALL0     =>   hall0,
		HALL1     =>   hall1,
		HALL2     =>   hall2,
		HALL3     =>   hall3,
		HALL4     =>   hall4,
		HALL5     =>   hall5,
		HALL6     =>   hall6,
		HALL7     =>   hall7  
      );


  programROM : entity usbrom 
  port map(
		addr => iabus,
		clk  => fclk,
		din  => x"0000",
		dout => idbus,
		we	=> '0'
	 );

  DataRam : entity usbram 
  port map(
		addra => pagedmwadd,
		addrb => pagedmradd,
		clk  => fclk,
		dina  => mobus,
--		douta => 
		doutb => mibus_ram,
		wea	=> pagedmwrite
	 );
	 
	MiscProcFixes : process (fclk)		-- need to match BlockRAM address pipeline register for I/O
	begin									 		-- and map memory/IO so 1K IO,1K memory, 1K IO, 1K memory
		if rising_edge(fclk) then
			ioradd <= mradd;
		end if;
		pagedmradd <= mradd(11) & mradd(9 downto 0);
		pagedmwadd <= mwadd(11) & mwadd(9 downto 0);
		pagedmwrite <= mwrite and mwadd(10);
	end process;		
	
	ram_iomux : process (ioradd(10),mibus_ram,mibus_io)
	begin
		if ioradd(10) = '1' then
			mibus <= mibus_ram;
		else
			mibus <= mibus_io;
		end if;
	end process;

	iodecode: process(ioradd,mwadd,mwrite)
	begin
		rseladd <= ioradd(7 downto 0);
		wseladd <= mwadd(7 downto 0);
		

		if wseladd = x"7B" and wiosel = '0' and mwrite = '1'then
			LoadUSBControlReg <= '1';
		else
			LoadUSBControlReg <= '0';		
		end if;	

		if rseladd = x"7B" and riosel = '0' then
			ReadUSBStatus <= '1';
		else
			ReadUSBStatus <= '0';		
		end if;	
		
		if wseladd = x"7C" and wiosel = '0' and mwrite = '1'then
			LoadUSBDataReg <= '1';
		else
			LoadUSBDataReg <= '0';		
		end if;	

		if rseladd = x"7C" and riosel = '0' then
			ReadUSBData <= '1';
		else
			ReadUSBData <= '0';		
		end if;	

		if wseladd = x"7D" and wiosel = '0' and mwrite = '1'then
			LoadSPICS <= '1';
		else
			LoadSPICS <= '0';		
		end if;	

		if rseladd = x"7D" and riosel = '0' then
			ReadSPICS <= '1';
		else
			ReadSPICS <= '0';		
		end if;	

		if wseladd = x"7E" and wiosel = '0' and mwrite = '1'then
			LoadSPIReg <= '1';
		else
			LoadSPIReg <= '0';		
		end if;	

		if rseladd = x"7E" and riosel = '0' then
			ReadSPIReg <= '1';
		else
			ReadSPIReg <= '0';		
		end if;	
		
		if wseladd = x"7F" and wiosel = '0' and mwrite = '1' then
			ReconfigSel <= '1';
		else
			ReconfigSel <= '0';		
		end if;	

	end process iodecode;
	
	doreconfig: process (fclk,ReConfigreg)
	begin
		if rising_edge(fclk) then
			if ReconfigSel = '1' then
				if mobus = x"5A" then
					ReConfigreg <= '1';
				end if;
			end if;
		end if;		
		RECONFIG <= not ReConfigreg;
	end process doreconfig;
	
	SoftDMCInterfaceShim: process (fclk)
	begin
		SDMCRAddr <= ioradd(3 downto 2);
		SDMCWAddr <= mwadd(3 downto 2);
		SDMCDataIn(15 downto 8) <= mobus;
		SDMCDatain(7 downto 0) <= SDMCLowDataLatch;
		
		if ioradd(11 downto 4) = x"08" then
			SDMCTSEn <= '1';
		else
			SDMCTSEn <= '0';
		end if;
		
		if mread = '1' and ioradd(11 downto 4) = x"08" and ioradd(0) = '1'  then
			SDMCReadStb <= '1';
		else
			SDMCReadStb <= '0';
		end if;				
		
		
		if mwadd(11 downto 4) = x"08"  and mwadd(0) = '1' and mwrite = '1' then
		-- write to SoftDMC on high byte
			SDMCWriteStb <= '1';		
		else
			SDMCWriteStb <= '0';
		end if;	
		
		mibus_io <= "ZZZZZZZZ";	
		if SDMCTSEn = '1' then
			if ioradd(0) = '0' then
				mibus_io <=	SDMCDataOut(7 downto 0);
			else
				mibus_io <= SDMCDataOut(15 downto 8);
			end if;
		end if;		
		
		if rising_edge(fclk) then
			-- save low data for merging together with high byte on writes 
			if mwadd(11 downto 4) = x"08" and mwadd(0) = '0' and mwrite = '1' then
				SDMCLowDataLatch <= mobus;
			end if;
			if SDMCWriteStb = '1' then
--				LEDLatch <= mobus;
			end if;	
		end if;
	end process;	
	
	USBInterfaceDrive: process (USBDataReg, USB_DATABUS, USB_TSEn, ReadUSBData, ReadUSBStatus)
	begin
		
		USB_DATABUS <= "ZZZZZZZZ";
		if USB_TSEn = '1' then 
			USB_DATABUS <= USBDataReg;
		end if;
		
		mibus_io <= "ZZZZZZZZ";
		if ReadUSBData = '1' and ReadUSBStatus = '0' then 
			mibus_io <= USB_DATABUS;
		end if;

		if ReadUSBStatus = '1' and ReadUSBData = '0' then 
			mibus_io(0) <= USB_RXF;			-- active low receiver has data
			mibus_io(1) <= USB_TXE;			-- active low xmit buffer has space
			mibus_io(7 downto 2) <= "101010";
		end if;
		
		if rising_edge(fclk) then
			if LoadUSBControlReg = '1' then
				USBContReg <= mobus(2 downto 0);
			end if;
			
			if LoadUSBDataReg = '1' then
				USBDataReg <= mobus;
			end if;

		end if;
		USB_RD <= USB_RdReg;
		USB_WRITE <= USB_WriteReg;
		LEDS <= SDMCLEDs;
--		LEDS <= not LEDLatch;
		PARACONFIG <= '0';
	end process USBInterfaceDrive;	

		doi3xpinout: if I30Pinout generate
			ConnectIOBITSI30: process(PWMA,DIRA,ENA,IDX,A,B,IOBITS)
															
			begin					
					for i in StartingI30Conn to StartingI30Conn + ConnsWithI30 -1 loop
					-- Note for 7I29/7I30/7I33/7I40  --
					B(4*i+1) <= IOBITS(PortWidth*i+0);
					A(4*i+1) <= IOBITS(PortWidth*i+1);
					B(4*i+0) <= IOBITS(PortWidth*i+2);
					A(4*i+0) <= IOBITS(PortWidth*i+3);
					IDX(4*i+1) <= IOBITS(PortWidth*i+4);
					IDX(4*i+0) <= IOBITS(PortWidth*i+5);
					IOBITS(PortWidth*i+6) <= PWMA(4*i+1);					
					IOBITS(PortWidth*i+7) <= PWMA(4*i+0);
					IOBITS(PortWidth*i+8) <= DIRA(4*i+1);					
					IOBITS(PortWidth*i+9) <= DIRA(4*i+0);					
					IOBITS(PortWidth*i+10) <= ENA(4*i+1);					
					IOBITS(PortWidth*i+11) <= ENA(4*i+0);				
					B(4*i+3) <= IOBITS(PortWidth*i+12);					
					A(4*i+3) <= IOBITS(PortWidth*i+13);					
					B(4*i+2) <= IOBITS(PortWidth*i+14);				
					A(4*i+2) <= IOBITS(PortWidth*i+15);
					IDX(4*i+3) <= IOBITS(PortWidth*i+16);
					IDX(4*i+2) <= IOBITS(PortWidth*i+17);
					IOBITS(PortWidth*i+18) <= PWMA(4*i+3);					
					IOBITS(PortWidth*i+19) <= PWMA(4*i+2);					
					IOBITS(PortWidth*i+20) <= DIRA(4*i+3);					
					IOBITS(PortWidth*i+21) <= DIRA(4*i+2);
					IOBITS(PortWidth*i+22) <= ENA(4*i+3);
					IOBITS(PortWidth*i+23) <= ENA(4*i+2);
				end loop;
			end process;
		end generate;	
		
		TwoIOPorts: if ConnsWithIO = 1 generate
			ConnectSDMCIO2: process(portadata,portbdata,IOBITS)
			begin
				IOBITS(StartingIOConn*PortWidth +11 downto StartingIOConn*PortWidth) <= portadata;
				IOBITS(StartingIOConn*PortWidth +23 downto StartingIOConn*PortWidth +12) <= portbdata;			
			end process;
		end generate;			
end Behavioral;
